@@ -5,66 +5,96 @@ use warnings;
 
 use EventStore::Tiny::Logger;
 use EventStore::Tiny::Event;
-use EventStore::Tiny::DataEvent;
+use EventStore::Tiny::TransformationStore;
 use EventStore::Tiny::EventStream;
 use EventStore::Tiny::Snapshot;
 
 use Clone qw(clone);
-use Storable;
-use Data::Compare; # Exports Compare()
+use IO::File;
+use YAML::Syck;     # Exports Dump(), Load()
+use Data::Compare;  # Exports Compare()
 
-# Enable handling of CODE refs (as event actions are code refs)
-$Storable::Deparse  = 1;
-$Storable::Eval     = 1;
-
-our $VERSION = '0.51';
+our $VERSION = '0.6';
 
 use Class::Tiny {
-    registry        => sub {{}},
     events          => sub {EventStore::Tiny::EventStream->new(
                              logger => shift->logger)},
+    trans_store     => sub {EventStore::Tiny::TransformationStore->new},
     init_data       => sub {{}},
     logger          => sub {EventStore::Tiny::Logger->log_cb},
     slack           => 0, # Default: strict mode
     cache_distance  => 0, # Default: store snapshot each time. no caching: undef
 }, '_cached_snapshot';
 
-# Class method to construct
-sub new_from_file {
-    my (undef, $fn) = @_;
-    return retrieve($fn);
+sub import_events {
+    my ($self, $fn) = @_;
+
+    # Rerieve
+    my $file    = IO::File->new($fn, 'r');
+    my $yaml    = do {local $/ = undef; <$file>};
+    my $events  = Load $yaml;
+    $file->close;
+
+    # Create
+    my $stream  = EventStore::Tiny::EventStream->new;
+    for my $data (@$events) {
+        $stream->add_event(EventStore::Tiny::Event->new(
+            uuid        => $data->{uuid},
+            timestamp   => $data->{timestamp},
+            name        => $data->{name},
+            trans_store => $self->trans_store,
+            data        => $data->{data},
+        ));
+    }
+
+    # Done
+    $self->events($stream);
 }
 
-sub store_to_file {
+sub export_events {
     my ($self, $fn) = @_;
-    return store($self, $fn);
+
+    # Simplify
+    my @events = ();
+    for my $event (@{$self->events->events}) {
+        push @events, {
+            uuid        => $event->uuid,
+            timestamp   => $event->timestamp,
+            name        => $event->name,
+            data        => $event->data,
+         };
+    }
+
+    # Export
+    my $file = IO::File->new($fn, 'w');
+    print $file Dump(\@events);
+    $file->close;
 }
 
 sub register_event {
     my ($self, $name, $transformation) = @_;
 
-    return $self->registry->{$name} = EventStore::Tiny::Event->new(
-        name            => $name,
-        transformation  => $transformation,
-        logger          => $self->logger,
-    );
+    # Register transformation
+    $self->trans_store->set($name => $transformation);
 }
 
 sub event_names {
     my $self = shift;
-    return [sort keys %{$self->registry}];
+    return [$self->trans_store->names];
 }
 
 sub store_event {
     my ($self, $name, $data) = @_;
 
-    # Lookup template event
-    my $template = $self->registry->{$name};
-    die "Unknown event: $name!\n" unless defined $template;
+    # Lookup event type
+    die "Unknown event: $name!\n"
+        unless defined $self->trans_store->get($name);
 
-    # Specialize event with new data
-    my $event = EventStore::Tiny::DataEvent->new_from_template(
-        $template, $data
+    # Create event
+    my $event = EventStore::Tiny::Event->new(
+        name        => $name,
+        trans_store => $self->trans_store,
+        data        => $data,
     );
 
     # Done
@@ -147,6 +177,8 @@ sub is_correct_snapshot {
 
 EventStore::Tiny - A minimal event sourcing framework.
 
+B<ATTENTION>: as of version 0.6 there is an API change regarding serialization which breaks existing applications that use serialization as well as their existing event stores.
+
 =begin html
 
 <p>
@@ -261,17 +293,17 @@ A subref (callback) which will be called each time an event is applied to the st
 
 =back
 
-=head3 new_from_file
+=head3 import_events
 
-    my $store = EventStore::Tiny->new_from_file($filename);
+    $store->import_events($filename);
 
-Deserializes an existing store object which was L</store_to_file>d before.
+Loads events from a file which was written by L</export_events> before. It replaces an existing event stream in C<$store>. Note: before using the resulting events, the event types need to be registered as the transformations are only referenced.
 
-=head3 store_to_file
+=head3 export_events
 
-    $store->store_to_file($filename);
+    $store->export_events($filename);
 
-Serializes the store object to the file system. It can be deserialized via L</new_from_file> later.
+Serializes the event stream to the file system. It can be imported back via L</import_events> later.
 
 =head2 EVENT SOURCING WORKFLOW
 
@@ -306,19 +338,19 @@ Returns a L<EventStore::Tiny::Snapshot> object which basically consists of the c
 
     my $types = $store->event_names;
 
-Returns an arrayref containing all event type names of registered events, sorted by name. These names are the values of L</registry>.
-
-=head3 registry
-
-    my $user_added = $store->registry->{UserAdded};
-
-Returns a hashref with event type names as keys and event types as values, which are L<EventStore::Tiny::Event> instances. Should be manipulated by L</register_event> only.
+Returns an arrayref containing all event type names of registered events, sorted by name.
 
 =head3 events
 
     my $event_stream = $store->events;
 
-Returns the internal L<EventStore::Tiny::EventStream> object that stores all concrete events (L<EventStore::Tiny::DataEvent> instances). Should be manipulated by L</store_event> only. Events should never be changed or removed.
+Returns the internal L<EventStore::Tiny::EventStream> object that stores all concrete events (L<EventStore::Tiny::Event> instances). Should be manipulated by L</store_event> only. Events should never be changed or removed.
+
+=head3 trans_store
+
+    my $transformation_store = $store->trans_store
+
+Returns the internal L<EventStore::Tiny::TransformationStore> object that stores all transformation subroutines that events refer to. Should be manipulated by L</register_event> only. The transformation store should never be changed or removed.
 
 =head3 init_state
 
